use strict;


#Get Settings
  my ($path,$input,$output,$prefix,$appendForward,$leftTag,$rightTag,$detailLevel,$productSize,
    $optSize,$minSize,$maxSize,$optTM,$minTM,$maxTM,$diffTM,$optGC,
    $minGC,$maxGC,$maxPoly) = GetSettings();

    unless(-e $input)
    {
    	print("File containing the input sequences could not be found. DesignPrimer aborted\n");
    	print("Edit the file DesignPrimerInput.txt to specify an input file\n");
    	print("Also make sure that this file is in the same directory as the script\n");
    }
    unless(-e $path)
    {
    	print("Primer 3 could not be found. Process aborted\n");
		print("Edit the file DesignPrimerInput.txt to specify the position of Primer 3\n");
		print("Also make sure that this file is in the same directory as the script\n");
    }
    unless($productSize && $optSize && $minSize && $maxSize && $optTM && $minTM && $maxTM && $diffTM && $optGC && $minGC && $maxGC && $maxPoly)
    {
    	print("Some parameters for Primer 3 are not valid\n");
    	print "Edit the file DesignPrimerInput.txt and specify valid parameters for Primer3\n";
    	print "if uncertain check the Primer 3 manual\n";
    }

#open fasta File
my($inputFH);
open($inputFH,$input);

#open outputFile
my $OUTPUT;
open($OUTPUT,">$output");

my $count=1;


my(@cont);
	while(@cont = GetNextFastaRecord($inputFH))
	{
		my($fastaID,$dna)=@cont;
		#If the records are not defined cancel the procedure;
		if(!defined($fastaID) || !defined($dna)) {last;}

        #Retrieve the microsatellite target information from the fasta ID
		my($targetStart,$targetLength)=ParseSciRoKoFastaID($fastaID);

        #Start primer3 and retrieve the information
		@cont=GetPrimer3Results($dna,$targetStart,$targetLength,$path,$productSize,
    $optSize,$minSize,$maxSize,$optTM,$minTM,$maxTM,$diffTM,$optGC,
    $minGC,$maxGC,$maxPoly);

		my($fwdPrim,$revPrim,$fwdTM,$revTM,$prodSize)=ParsePrimer3Results(@cont);

		if(defined($fwdPrim) && defined($revPrim))
		{
            print $OUTPUT $prefix.$count.$leftTag."\t".$appendForward.$fwdPrim."\t".$prefix.$count.$rightTag."\t".$revPrim;

            if($detailLevel)
            {
            	print $OUTPUT "\tTM_left:\t$fwdTM\tTM_right:\t$revTM\tPCR_product_size:\t$prodSize\tFull_fasta_ID:\t$fastaID";
            }



            print $OUTPUT "\n";
			$count++;
		}
		else
		{
			print "Could not find primer for: $fastaID\n";
		}

	}


close($inputFH);
close($OUTPUT);


          $count--;
   print "Successfully finished primer design\n";
   print "$count primer pairs have been designed\n";
exit;


sub ParsePrimer3Results
{
my(@content)=@_;
my($fwdPrim,$revPrim,$fwdTM,$revTM,$prodSize);
my $cont=join("\n",@content);

	($fwdPrim)=$cont=~m/PRIMER_LEFT_SEQUENCE=(\S+)\s*\n/s;
	($revPrim)=$cont=~m/PRIMER_RIGHT_SEQUENCE=(\S+)\s*\n/s;
	($fwdTM)=$cont=~m/PRIMER_LEFT_TM=(\S+)\s*\n/s;
	($revTM)=$cont=~m/PRIMER_RIGHT_TM=(\S+)\s*\n/s;
	($prodSize)=$cont=~m/PRIMER_PRODUCT_SIZE=(\S+)\s*\n/s;
    return ($fwdPrim,$revPrim,$fwdTM,$revTM,$prodSize);
}

sub GetPrimer3Results
{
my($dna,$targetStart,$targetLength,$path,$productSize,
    $optSize,$minSize,$maxSize,$optTM,$minTM,$maxTM,$diffTM,$optGC,
    $minGC,$maxGC,$maxPoly)=@_;

    my $temp=reverse($path);
    $temp=~m/\\/g;
	my $intTemp=pos($temp);
	$temp=substr($temp,$intTemp);
	$temp=reverse($temp);
    $temp.="\\";
    $temp.="temp.txt";
    my $OUTPUT;
    open($OUTPUT,">$temp");
    print $OUTPUT "PRIMER_SEQUENCE_ID=temp\n";
    print $OUTPUT "SEQUENCE=$dna\n";
    print $OUTPUT "TARGET=$targetStart,$targetLength\n";
    print $OUTPUT "PRIMER_PRODUCT_SIZE_RANGE=$productSize\n";
    print $OUTPUT "PRIMER_OPT_SIZE=$optSize\n";
    print $OUTPUT "PRIMER_MIN_SIZE=$minSize\n";
    print $OUTPUT "PRIMER_MAX_SIZE=$maxSize\n";
    print $OUTPUT "PRIMER_OPT_TM=$optTM\n";
    print $OUTPUT "PRIMER_MIN_TM=$minTM\n";
    print $OUTPUT "PRIMER_MAX_TM=$maxTM\n";
    print $OUTPUT "PRIMER_MAX_DIFF_TM=$diffTM\n";
    print $OUTPUT "PRIMER_OPT_GC_PERCENT=$optGC\n";
    print $OUTPUT "PRIMER_MIN_GC_PERCENT=$minGC\n";
    print $OUTPUT "PRIMER_MAX_GC_PERCENT=$maxGC\n";
    print $OUTPUT "PRIMER_MAX_POLY_X=$maxPoly\n";
    print $OUTPUT "PRIMER_NUM_RETURN=1\n";
    print $OUTPUT "=\n";
    close($OUTPUT);

    my @content = `$path < $temp`;
    return @content;


}

sub ParseSciRoKoFastaID
{
	my($fastaID)=@_;
	my($targetStart,$targetLength);
	($targetStart)=$fastaID=~/Startpos_here=(\S*)/;  #Startpos_here=201  Length=20
	($targetLength)=$fastaID=~/Length=(\S*)/;

	return($targetStart,$targetLength);
}



sub GetNextFastaRecord
{
 use strict;
	my($HANDLE)=@_;
	my($defaultEnd)=$/;
	$/=">";
	my $line;

	do
	{
	   $line=<$HANDLE>;
	   unless($line){return;}

	}
    while($line=~/^\s*>\s*$/s);

    my($id,$sequence)=$line=~ /^(.*)\n((.*\n)*.*)/m;
    $sequence=~s/[\s>]//g;

    $/=$defaultEnd;
    return ($id,$sequence);

}

sub GetSettings
{
	use strict;
	open(SETTINGS,'.\\DesignPrimerInput.txt');
	my @content=<SETTINGS>;
	my $cont=join("\n",@content);
	my($path,$input,$output,$prefix,$appendForward,$productSize);
	my($optSize,$minSize,$maxSize,$optTM,$minTM,$maxTM,$diffTM,$optGC,$minGC,$maxGC);
	my($maxPoly,$leftTag,$rightTag,$detailLevel);

	($path)=$cont=~m/PATH=(\S+)\s*\n/s;
	($input)=$cont=~m/INPUT_FILE=(\S+)\s*\n/s;
	($output)=$cont=~m/OUTPUT_FILE=(\S+)\s*\n/s;
	($prefix)=$cont=~m/PREFIX=(\S+)\s*\n/s;
	($appendForward)=$cont=~m/APPEND_TO_FORWARD_PRIMER=(\S+)\s*\n/s;
	($productSize)=$cont=~m/PRIMER_PRODUCT_SIZE_RANGE=(\S+)\s*\n/s;
	($optSize)=$cont=~m/PRIMER_OPT_SIZE=(\S+)\s*\n/s;
	($minSize)=$cont=~m/PRIMER_MIN_SIZE=(\S+)\s*\n/s;
	($maxSize)=$cont=~m/PRIMER_MAX_SIZE=(\S+)\s*\n/s;
	($optTM)=$cont=~m/PRIMER_OPT_TM=(\S+)\s*\n/s;
	($minTM)=$cont=~m/PRIMER_MIN_TM=(\S+)\s*\n/s;
	($maxTM)=$cont=~m/PRIMER_MAX_TM=(\S+)\s*\n/s;
	($optGC)=$cont=~m/PRIMER_OPT_GC_PERCENT=(\S+)\s*\n/s;
	($minGC)=$cont=~m/PRIMER_MIN_GC_PERCENT=(\S+)\s*\n/s;
	($maxGC)=$cont=~m/PRIMER_MAX_GC_PERCENT=(\S+)\s*\n/s;
	($maxPoly)=$cont=~m/PRIMER_MAX_POLY_X=(\S+)\s*\n/s;
	($diffTM)=$cont=~m/PRIMER_MAX_DIFF_TM=(\S+)\s*\n/s;
	($leftTag)=$cont=~m/LEFT_PRIMER_TAG=(\S+)\s*\n/s;
	($rightTag)=$cont=~m/RIGHT_PRIMER_TAG=(\S+)\s*\n/s;
	($detailLevel)=$cont=~m/DETAIL_LEVEL=(\S+)\s*\n/s;



    return ($path,$input,$output,$prefix,$appendForward,$leftTag,$rightTag,$detailLevel,$productSize,
    $optSize,$minSize,$maxSize,$optTM,$minTM,$maxTM,$diffTM,$optGC,
    $minGC,$maxGC,$maxPoly);


}
